@@ -143,15 +143,15 @@ dist <- setRefClass(
       family <- getSelection(tbbox1$family)
       colour <- character(0)
       save   <- tclvalue(tbbox1$goption$value[[1]])
-      theme  <- checkTheme(tclvalue(tbbox1$theme$value))
-
+      theme  <- checkTheme(getSelection(tbbox1$theme))
+      
       options(
         kmg2FontSize   = tclvalue(tbbox1$size$value),
         kmg2FontFamily = seq_along(tbbox1$family$varlist)[tbbox1$family$varlist == getSelection(tbbox1$family)] - 1,
         kmg2SaveGraph  = tclvalue(tbbox1$goption$value[[1]]),
-        kmg2Theme      = tclvalue(tbbox1$theme$value)
+        kmg2Theme      = seq_along(tbbox1$theme$varlist)[tbbox1$theme$varlist == getSelection(tbbox1$theme)] - 1
       )
-
+      
       funcType <- tclvalue(rbbox1$value)
       if (funcType == "1") {
         yauto <- "Density"
